/**
 * @file Implémentation des requêtes GraphQL.
 * @author akka vodol
*/
import _ from 'lodash';
import { assertBinaryExpression } from 'babel-types';
import knex from '../../db/knex_router';

import passport from 'passport';
import './../config_passport';

import * as connectors from './connectors/connectors';
import * as list_selectors from './connectors/list_selectors';
import * as authentifiers from './connectors/authentifiers';

/** 
 * @description Résolveurs des différentes requêtes GraphQL
*/

export const resolvers = {
    // @rights user
    Query: {

        // group queries

        allGroups: async function(obj, args, context){
            let user = await authentifiers.anonymous(context.bindUser);
            return user && connectors.getAllVisibleGroups(user);
        },

        allSimpleGroups: async function (obj, args, context){
            let user = await authentifiers.anonymous(context.bindUser);
            return user && connectors.getAllVisibleSimpleGroups(user);
        },

        group: async function(obj, args, context){
            let user = await authentifiers.anonymous(context.bindUser);
            return user && connectors.getGroupIfVisible(user, args.uid);
        },

        simpleGroup: async function(obj, args, context){
            let user = await authentifiers.anonymous(context.bindUser);
            return user && connectors.getSimpleGroupIfVisible(user, args.uid);
        },
        metaGroup: async function(obj, args, context){
            let user = await authentifiers.anonymous(context.bindUser);
            return user && connectors.getMetaGroupIfVisible(user, args.uid);
        },

        /*
         * Message queries.
         */
        allPosts: function(obj, args, context) {
            return knex.select().from("posts");
        },

        post: function (obj, args, context) {
            return knex.select().from("posts").where('id',args.id).then(res => res[0]);
        },

        allAnnouncements: function(obj, args, context) {
            return knex.select().from("announcements");
        },

        allEvents: function(obj, args, context) {
            return knex.select().from("events");
        },


        // user queries

        user: async function(obj, args, context){
            let user = await authentifiers.anonymous(context.bindUser);
            return user && connectors.getUser(user,args.uid);
        },

        searchTOL: (obj, args, context) => {
            const result = connectors.utilisateur.repliquerTOLdesIds({
                givenName: args.givenName,
                lastName: args.lastName,
                nickname: args.nickname,
                nationality: args.nationality,
                school: args.school,
                promotion: args.promotion,
                groups: args.groups,
                studies: args.studies,
                sport: args.sport,
                phone: args.phone,
                mail: args.mail,
                adress: args.adress,
                ip: args.ip
            });
            return result;
        },

        // viewer queries

        // member queries

        allMembers : async function(obj, args, context){
            let user = await authentifiers.member(context.bindUser, args.from);
            return user && connectors.getGroupMemberUsers(context.bindUser, obj.groupUID);
        },

        // speaker queries

        allRequests: async function(obj, args, context){
            let res = [];
            let user = authentifiers.admin(context.bindUser, args.from);
            if(user){
                res = res.concat(await connectors.getUserJoinGroupRequests(user, args.from));
            }
            user = user || authentifiers.speaker(user, args.from);
            if(user){
                res = res.concat(await connectors.getGroupJoinEventRequests(user, args.from));
                res = res.concat(await connectors.getYourGroupHostEventRequests(user, args.from));
            }
            return res;
        }

    },


    Request : {
        __resolveType : function(obj){
            return obj.type;
        }
    },

    // @rights admin(obj.groupUID)
    UserJoinGroup: {
        user : (obj, args, context) => {
            return connectors.getUser(context.bindUser, obj.useruid);
            /*return connectors.getUser(context.bindUser, "quentin.gendre");
            if(obj.useruid === "anatole.romon"){
                return connectors.getUser(context.bindUser, "anatole.romon").then(res => {
                    return connectors.getUser(context.bindUser, "quentin.gendre");
                });
            }else{
                return new Promise( (resolve, reject) => {
                    resolve({givenName : "patrick"});
                });
            }*/
        }
    },

    // @rights speaker(obj.groupUID)
    GroupJoinEvent : {
        event: (obj, args, context) => {
            return connectors.getEvent(context.bindUser, obj.eventuid);
        },
        groupWantingToJoin: (obj, args, context) => {
            return connectors.getGroup(context.bindUser, obj.senderuid);
        }
    },

    // @rights speaker(obj.groupUID)
    YourGroupHostEvent : {
        event: (obj, args, context) => {
            return connectors.getEvent(context.bindUser, obj.eventuid);
        },
        sender: (obj, args, context) => {
            return connectors.getGroup(context.bindUser, obj.senderuid);
        }
    },

    // @rights user
    User : {
        groups : (obj, args, context) => {
            let result = Promise.all(obj.groups.map((grid) => {
                return connectors.getSimpleGroup(context.bindUser,grid);
            }));

            return result.then(groups => {
                return _.filter(groups,(o) => !_.isUndefined(o));
            });
        },

    },

    // @rights user
    Mutation: {

        // Superviser mutations

        takeAdminRights : async function(obj, args, context){
            let user = await authentifiers.superviser(user, args.from);
            return user && await connectors.takeAdminRights(user, args.from, user.justification);
        },

        releaseAdminRights : async function(obj, args, context){
            let user = await authentifiers.superviser(user, args.from);
            return user && await connectors.releaseAdminRights(user, args.from, user.justification);
        },

        // Admin mutations

        createSubgroup: async function (obj, args, context){
            let user = authentifiers.admin(context.bindUser, args.from);
            return user && connectors.createSubgroup(user, args);
        },

        login: async function(obj, args, context) {

            passport.authenticate('ldapauth', (err, user, info) => {

                context.request.login(user, (err) => {
                    console.log(err);
                });

            });
            
            return "Toubi";
        }

    },

    Message: {

        __resolveType: function(obj) {
            return obj.type;
        }

    },

    Post: {
        authors: (obj, args, context) => {
            return knex.select().from('groups').whereIn('uid', obj.authors);
        }
    },

    Announcement: {
        forEvent : function(obj, args, context){
            // le champ is_announcement n'existe que sur les Events
            // une ligne de la bdd events peut résoudre comme évènement ou comme annonce
            if(obj.is_announcement) 
                return obj;
            else
                return null;
        }
    },

    Event: {

        startTime : function(obj){
            return obj.start_time;
        },

        endTime : function(obj){
            return obj.end_time;
        },

        asAnnouncement : function(obj, args, context){
            // le champ is_announcement indique si il existe une annonce qui va avec l'évènement
            // une ligne de la bdd events peut résoudre comme évènement ou comme annonce
            if(obj.is_announcement) 
                return obj;
            else
                return null;
        }
    },

    // @rights viewer(obj.uid)
    Group: {
        __resolveType: async (obj) => {
            
            switch(obj.type) {
            case "simple":
                return "SimpleGroup";
            case "meta":
                return "MetaGroup";
            }
        }
    },

    // @rights viewer(obj.uid)
    SimpleGroup: {
        
        members: (obj, args, context) => {
            return connectors.utilisateur.listMembers(context.bindUser,obj.uid);
        }
    },

    // @rights viewer(obj.uid)
    MetaGroup: {
        members: (obj, args, context) => {
        }
    }
};
